import { Component, OnInit, Output, EventEmitter, Input, ContentChild, ElementRef, OnChanges, DoCheck, SimpleChanges, AfterContentInit, AfterContentChecked, AfterViewInit, AfterViewChecked } from '@angular/core';
import { User } from '../models/user.model';
import { UsersService } from '../services/users.service';
import { NgModel, NgForm, FormGroup, FormControl, Validators, FormArray, ValidatorFn, AbstractControlOptions, ValidationErrors, FormBuilder } from '@angular/forms';
import { Observable } from 'rxjs';
import { delay } from 'rxjs/operators';
import { MyValidators } from '../myValidators/myValidators';

@Component({
  selector: 'app-user-form',
  templateUrl: './user-form.component.html',
  styleUrls: ['./user-form.component.sass'],
  styles: [
    `
    input.ng-touched.ng-invalid {border: solid red 3px}
    input.ng-touched.ng-valid {border: solid green 3px}
    `
  ]
})
export class UserFormComponent implements OnInit {



 
  validateName(usersName: FormControl): Observable<ValidationErrors> {
    return  new Observable<ValidationErrors>(observer => {
      let users: User[] = this.userService.getUsers();

      let user : User = users.find(user => user.name === usersName.value);

      if(user) {
        observer.next({
          nameError: true
        });
        observer.complete();
      }
      observer.next(null);
      observer.complete();


    });
  }

  
  
  linkPhoneValid(): ValidatorFn | ValidatorFn[] | AbstractControlOptions {
    return (group: FormGroup):ValidationErrors => {

      let link = group.controls['link']; 
      let phone = group.controls['phoneNumber'];
      
      if(!link || !phone) {
          return null;
      }

      if((!link.value && !phone.value) || (link.value && phone.value)) {
        return {'linkphone' : true};
      }

      return null;
    }
  }


  @Output() onUserAdded: EventEmitter<User> = new EventEmitter<User>(); 
  @Output() nameChange:EventEmitter<string> = new EventEmitter<string>();

  @Input() name: string;

  age:number = 45;

  user: User;

  isWarning: boolean = true;

  title: string = "Форма добавления пользователя";

  @ContentChild('fromParent')
  fromParent: ElementRef;

  userForm: FormGroup = new FormGroup({
    "name" : new FormControl("John",[
        Validators.required,
        Validators.maxLength(10),
        Validators.minLength(3),
        MyValidators.nameValid
    ], 
    [
      this.validateName.bind(this)
    ]
    ),
    "age" : new FormControl("",[
        Validators.required
    ]),

    "account" : new FormGroup({
      "email" : new FormControl("",[
          Validators.required, Validators.email
      ]),
      "password" : new FormControl("",[
          Validators.required
      ]),
    }),
    "social" : new FormGroup({
      "link" : new FormControl("",[]),
      "phoneNumber" : new FormControl("",[]),
    }, this.linkPhoneValid()),

    "phones" : new FormArray([
      new FormControl("+7", Validators.required)
    ])


  });


  form2: FormGroup;
  constructor(private userService: UsersService, private formBuilder: FormBuilder) { 
    this.user =  new User('John',15);

    this.form2 = formBuilder.group({
      "name" : ["John", [
        Validators.required,
        Validators.maxLength(10),
        Validators.minLength(3)
      ]
      ]
    });

  }
  ngOnInit(): void {
  
  }

  


  save(event, name: NgModel, age: NgModel) {
    
    let userTmp: User = new User(name.value, age.value);
    this.onUserAdded.emit(userTmp);

    this.fromParent.nativeElement.textContent = "Hello Parent";
    
  }


  onSubmit() {



    console.log(this.userForm.valid);
    
    this.userForm.reset();
    

  }

  

  addPhone() {
    (<FormArray>this.userForm.get('phones')).push(new FormControl("+7", Validators.required));
  }

  

}
